#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "ptrbuf.h"
#include "strbuf.h"
#include "expr.h"
#include "widget.h"
#include "gfx.h"
#include "edit_attr.h"
#include "show_resources.h"

static int enumindex(char *K,int V)
{
  char *Z;
  Z= strchr(K,V);
  if (Z) return Z-K;
  return 0;
}

static int show_text_field(field_t *F, char *name, int widget)
{
  if (strcmp(F->name,name)) return 0;
  gfx_set_text(widget, F->data);
  return 1;
}

static void show_field(field_t *F)
{
  if (!strcmp(F->name,"iconpos"))
  {
    char *v;
    v= F->data;
    gfx_set_selected(WGiconpos, enumindex("LRTB", v[0]));
    return ;
  }
  if (show_text_field(F,"text",WGtext)
      || show_text_field(F,"icon", WGicon)
      || show_text_field(F,"fg", WGfg)
      || show_text_field(F,"bg", WGbg)
      || show_text_field(F,"group", WGgroup)
      || show_text_field(F,"init_start", WGstart)
      || show_text_field(F,"init_end", WGend)
      || show_text_field(F,"init_step", WGstep)
      || show_text_field(F,"n_subs", WGsubs))
    return ;
  
  if (!strcmp(F->name,"vertical"))
  {
    char *v;
    v= F->data;
    gfx_set_state(WGvertical, v[0]=='1');
    return ;
  }
  if (!strcmp(F->name,"just"))
  {
    char *v;
    v= F->data;
    gfx_set_selected(WGjust, enumindex("LCR", v[0]));
    return ;
  }
  if (!strcmp(F->name,"font"))
  {
    char *v;
    v= F->data;
    gfx_set_selected(WGfont, enumindex("123", v[0]));
    return ;
  }
  printf("unknown attrib: %s\n",F->name);
}

static void show_attribs(Uwidget_t *W)
{
  int i;
  field_t *F;
  for(i=0;i<W->klass->nfields;i++)
     show_field(W->klass->fields+i);
  for(F=W->fields;F;F=F->next)
     show_field(F);
}

static field_t *find_widget_field(Uwidget_t *W,char *name)
{
  field_t *F;
  for(F=W->fields;F;F=F->next)
    if (!strcmp(F->name,name)) return F;
  return NULL;
}
static int apply_value(Uwidget_t *W,field_t *ref, char *S)
{
  field_t *F;
  F= find_widget_field(W, ref->name);
  
  if (F)
  {
    F->data= S;
    return 0;
  }
  if (S[0])
  {
    F= malloc(sizeof(*F));
    F->name= strdup(ref->name);
    F->type= ref->type;
    F->data= S;
    F->next= W->fields;
    W->fields= F;
  }
  return 0;
}

static int apply_tfield(Uwidget_t *W, field_t *ref, int ent)
{
  char *S;
  S= gfx_get_text(ent);
  return apply_value(W, ref, S);
}

static void apply_selfield
  (Uwidget_t *W,field_t *ref, int selw,int nent, char **ent)
{
  int K;
  K= gfx_get_selected(selw);
  if (K<0 || K>=nent) return ;
  apply_value(W, ref, strdup(ent[K]));
}

static int apply_field(Uwidget_t *W, field_t *ref)
{
  char *N;
  N= ref->name;
  if (!strcmp(N, "text")) return apply_tfield(W,ref,WGtext);
  if (!strcmp(N, "icon")) return apply_tfield(W,ref,WGicon);
  if (!strcmp(N, "fg")) return apply_tfield(W,ref,WGfg);
  if (!strcmp(N, "bg")) return apply_tfield(W,ref,WGbg);
  if (!strcmp(N, "group")) return apply_tfield(W,ref,WGgroup);
  if (!strcmp(N, "init_start")) return apply_tfield(W,ref,WGstart);
  if (!strcmp(N, "init_end")) return apply_tfield(W,ref,WGend);
  if (!strcmp(N, "init_step")) return apply_tfield(W,ref,WGstep);
  if (!strcmp(N, "n_subs")) return apply_tfield(W,ref,WGsubs);
  if (!strcmp(N, "iconpos"))
  {
     static char *icopo[]= { "T", "B", "L", "R" };
     apply_selfield(W, ref, WGiconpos, 4, icopo);
     return 0; 
  } 
  if (!strcmp(N, "just"))
  {
     static char *jus[]= { "L", "C", "R" };
     apply_selfield(W, ref, WGjust, 3, jus);
     return 0;
  }
  if (!strcmp(N, "font"))
  {
     static char *fon[]= { "1", "2", "3" };
     apply_selfield(W, ref, WGfont, 3, fon);
     return 0;
  }
  if (!strcmp(N,"vertical"))
  {
    int Z;
    Z= gfx_get_state(WGvertical);
    apply_value(W,ref, strdup(Z?"1":"0"));
    return ;
  }
}

static void get_attribs(Uwidget_t *W)
{
  int i;
  for(i=0;i<W->klass->nfields;i++)
     apply_field(W,W->klass->fields+i);
}


void edit_attribs(Uwidget_t *W)
{
  char *sv;
  char buf[200];

  gfx_SCRediwid();

  snprintf(buf,200,"Widget: %s Class: %s",W->ident,W->klass->ident);
  gfx_set_text(WGident, buf);

  gfx_insert(WGiconpos, -1, "Left", NULL,NULL);
  gfx_insert(WGiconpos, -1, "Right", NULL,NULL);
  gfx_insert(WGiconpos, -1, "Top", NULL,NULL);
  gfx_insert(WGiconpos, -1, "Bottom", NULL,NULL);

  gfx_insert(WGjust, -1, "Left", NULL, NULL);
  gfx_insert(WGjust, -1, "Center", NULL, NULL);
  gfx_insert(WGjust, -1, "Right", NULL, NULL);

  gfx_insert(WGfont, -1, "1", NULL, NULL);
  gfx_insert(WGfont, -1, "2", NULL, NULL);
  gfx_insert(WGfont, -1, "3", NULL, NULL);

  show_attribs(W);

  while(1)
  {
    gfx_event_t event;
    if (!gfx_event(NULL,&event)) continue;
    switch(event.type)
    {
    case WGok: 
       get_attribs(W);
    case GFXEV_QUIT: 
    case WGcancel:
       goto done;
    case WGiconSEL:
       sv= show_resources();
       if (sv) gfx_set_text(WGicon,sv);
       break;
    }
  }
done:
  gfx_leave_screen();
#if 0
  if (!strcmp(W->klass->ident,"Panel"))
    make_panel_subs(W);
#endif
}



/**

For some things like lists, dropdowns, etc. I need special attribute editors.
In fact, I should make separate attribute editors for all widgets. However,
making these with the old gfx library is quite cumbersome because of all the
positioning and naming code. I should make the layout editor using the
new library. 

For that, I need to bootstrap it from the old code to the new code. I can do
this by writing a new program in the new library and then communicating 
widget attributes etc. between the old and new libraries. Another option is
to design the whole thing from scratch. 

OK, so I will do the following: I should make a command line tool which
converts a given screen description into code. This will be my backup.
Afterwards, I can use this tool to do the conversion in case I mess up
the library etc.

**/
