#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "expr.h"
#include "widget.h"
#ifndef KLASSES_NO_GFX
#include "gfx.h"
#endif


static char *get_field(Uwidget_t *W, char *name, char *def)
{
  field_t *F;
  char *D;
  int i;
  Uklass_t *K;

  for(F=W->fields;F;F=F->next)
    if (!strcmp(F->name, name))
    {
      D= F->data;
      if (D && D[0]) return D;
      break;
    }

  K= W->klass;
  for(i=0;i<K->nfields;i++)
    if (!strcmp(K->fields[i].name,name))
    {
      D= K->fields[i].data;
      if (D && D[0]) return D;
      break;
    }

  return def;
}

static char *nustr(char *x)
{
  if (x) return x;
  else return "NULL";
}

static char *Cstr(char *x)
{
  char *R;
  int L;
  int i,j;
  char *legal= " abcdefghijklmnopqrstuvwxyz!@#$%^&*()_+=-[]{};:,./<>?`~"
               "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
  // char *hex= "0123456789abcdef";
  if (!x) return "NULL";
  L= strlen(x);
  R= malloc(4*L+2+1);
  j= 0;
  R[j++]= '"';
  for(i=0;x[i];i++)
   if (strchr(legal,x[i])) R[j++]= x[i];
   else { sprintf(R+j,"\\%03o", *(unsigned char*)(x+i)); j+= 4; }
  R[j++]= '"';
  R[j++]= 0;
  return R;
}

static void Kcreate_button(Uwidget_t *W, FILE *f)
{
  fprintf(f, "  widgets[%s]= gfx_button(%s, '%s', %s,GFXCLASS_BUTTON);\n",
        W->ident, Cstr(get_field(W,"text", NULL)),
                  get_field(W,"iconpos", "T"),
                  nustr(get_field(W,"icon",NULL))
  );
}

static void Kcreate_toggle(Uwidget_t *W, FILE *f)
{
  fprintf(f, "  widgets[%s]= gfx_button(%s, '%s', %s,GFXCLASS_TOGGLE);\n",
        W->ident, Cstr(get_field(W,"text", NULL)),
                  get_field(W,"iconpos", "T"),
                  nustr(get_field(W,"icon",NULL))
  );
}

static void Kcreate_label(Uwidget_t *W, FILE *f)
{
  fprintf(f, "  widgets[%s]= gfx_label(%s,%s,%s,'%s',%s);\n", 
             W->ident, 
    get_field(W,"font","1"),
    get_field(W,"fg", "0"),
    get_field(W,"bg", "1"),
    get_field(W,"just","C"),
    Cstr(get_field(W,"text",""))
  );
}

static void Kcreate_radio(Uwidget_t *W, FILE *f)
{
  fprintf(f, "  widgets[%s]= radio_new(%s,%s);\n", 
                W->ident, Cstr(get_field(W,"text","")),
                          get_field(W,"group","-1"));
}

static void Kcreate_checkbox(Uwidget_t *W, FILE *f)
{
  fprintf(f, "  widgets[%s]= checkbox_new(%s);\n", 
                W->ident, Cstr(get_field(W,"text","")));
}

static void Kcreate_scrollbar(Uwidget_t *W,FILE *f)
{
  fprintf(f, "  widgets[%s]= scrollbar_new(%s,%s,%s,%s,1,NULL,NULL);\n",
               W->ident, get_field(W,"vertical","0"),
                         get_field(W,"init_start","0.0"),
                         get_field(W,"init_end","1.0"),
                         get_field(W,"init_step","0.1") );
}

static void Kcreate_panel(Uwidget_t *W, FILE *f)
{
  int i,n;
  Uwidget_t *cw;
  Uscreen_t *S;
  fprintf(f,"  {\n"
            "    panel_t *PP; \n"
            "    widgets[%s]= (widget_t*) (PP= panel_new(%d));\n",
               W->ident,W->n_subs); 
  for(i=0;i<W->n_subs;i++)
  {
    
    fprintf(f,"    static int WL_%d[]= {\n", i);
    S= W->subs[i];
    for(n=0,cw=S->widgets;cw;cw=cw->next,n++) fprintf(f,"    %s,\n", cw->ident);
    fprintf(f,"    };\n");
    fprintf(f,"    PP->subs[%d].n_widgets= %d;\n", i, n);
    fprintf(f,"    PP->subs[%d].widgets= WL_%d;\n", i,i);
  }
  fprintf(f,"  }\n");
}

static void Kcreate_dropdown(Uwidget_t *W,FILE *f)
{
  fprintf(f,"  widgets[%s]= dropdown_new(1);\n",W->ident);
}

static void Kcreate_list(Uwidget_t *W,FILE *f)
{
  fprintf(f,"  widgets[%s]= gfx_list(0,0,1);\n",W->ident);
}

static void Kcreate_pixgrid(Uwidget_t *W,FILE *f)
{
  fprintf(f,"  widgets[%s]= gfx_pixgrid();\n",W->ident);
}

static void Kcreate_textentry(Uwidget_t *W,FILE *f)
{
  fprintf(f,"  widgets[%s]= text_entry_new(%s);\n",W->ident,
                       Cstr(get_field(W,"text","")) );
}

static void Kcreate_canvas(Uwidget_t *W,FILE *f)
{
   fprintf(f,"  widgets[%s]= gui_canvas_new();\n",W->ident);
}


static field_t fields_button[]= 
 {
   { "text", TYP_STR, NULL },
   { "iconpos", TYP_INT, "T" },
   { "icon", TYP_COLORIMG, NULL },
 };
 
static Uklass_t klass_button= { 
   "Button", 3, fields_button, 
   Kcreate_button };

static field_t fields_toggle[]=
 {
   { "text", TYP_STR, NULL },
   { "iconpos", TYP_INT, "T" },
   { "icon", TYP_COLORIMG, NULL },
 };
static Uklass_t klass_toggle= {
   "Toggle", 3, fields_toggle,
   Kcreate_toggle };

static field_t fields_label[]=
 {
   { "text", TYP_STR, NULL },
   { "fg",   TYP_INT, "0x000000" },
   { "bg",   TYP_INT, "0xededed" },
   { "font", TYP_INT, "1" },
   { "just", TYP_INT, "C" },
 };

static Uklass_t klass_label= {
   "Label", 5, fields_label, 
   Kcreate_label };

static field_t fields_radio[]= 
 {
   { "text", TYP_STR, NULL },
   { "group", TYP_INT, "-1"},
 };

static Uklass_t klass_radio= {
  "Radio", 2, fields_radio,
  Kcreate_radio };

static Uklass_t klass_canvas=
{
  "Canvas", 0, NULL, Kcreate_canvas,
};

static field_t fields_checkbox[]= 
 {
   { "text", TYP_STR, NULL },
 };

static Uklass_t klass_checkbox= {
  "CheckBox", 1, fields_checkbox,
  Kcreate_checkbox };

static field_t fields_panel[]= 
 {
   { "n_subs", TYP_INT, "2" }
 };

static Uklass_t klass_panel = {
  "Panel", 1, fields_panel,
  Kcreate_panel };

static field_t fields_scrollbar[]=
 {
   { "vertical", TYP_INT, "0" },
   { "init_start", TYP_INT, "0.0" },
   { "init_end", TYP_INT, "1.0" },
   { "init_step", TYP_INT, "0.1" },
 };

static Uklass_t klass_scrollbar=
 { "ScrollBar", 4, fields_scrollbar ,
   Kcreate_scrollbar };

static Uklass_t klass_dropdown=
 { "DropDown", 0, NULL ,
   Kcreate_dropdown };

static Uklass_t klass_list=
 { "List", 0, NULL ,
   Kcreate_list };

static Uklass_t klass_pixgrid=
 { "PixGrid", 0, NULL ,
   Kcreate_pixgrid };

static field_t fields_textentry[]=
 {
   { "text", TYP_STR, NULL },
 };

static Uklass_t klass_textentry= {
   "TextEntry", 1, fields_textentry, 
   Kcreate_textentry };


Uklass_t *klasses[]=
{ &klass_button,
  &klass_canvas,
  &klass_checkbox,
  &klass_dropdown,
  &klass_label,
  &klass_list,
  &klass_panel,
  &klass_pixgrid,
  &klass_radio,
  &klass_scrollbar,
  &klass_textentry,
  &klass_toggle,
  NULL
};


#ifndef KLASSES_NO_GFX
Uklass_t *choose_klass()
{
  Uklass_t *ret;
  gfx_event_t event;
  ret= NULL;
  gfx_SCRwidsel();
  while(1)
   if (gfx_event(NULL,&event))
   switch(event.type)
   {
   case WwselCANCEL:
   case GFXEV_QUIT:                             goto done;
   case WwselTOGGLE:    ret= &klass_toggle;     goto done;
   case WwselTEXTENTRY: ret= &klass_textentry;  goto done;
   case WwselSCROLLBAR: ret= &klass_scrollbar;  goto done;
   case WwselRADIO:     ret= &klass_radio;      goto done;
   case WwselPIXGRID:   ret= &klass_pixgrid;    goto done;
   case WwselPANEL:     ret= &klass_panel;      goto done;
   case WwselLIST:      ret= &klass_list;       goto done;
   case WwselLABEL:     ret= &klass_label;      goto done;
   case WwselDROPDOWN:  ret= &klass_dropdown;   goto done;
   case WwselCHECKBOX:  ret= &klass_checkbox;   goto done;
   case WwselCANVAS:    ret= &klass_canvas;     goto done;
   case WwselBUTTON:    ret= &klass_button;     goto done;
   }
done:
  gfx_leave_screen();
  return ret;
}
#endif
